/*
 * add a file/files to an sqlite db
 * in the 'files' table.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/mman.h>

#include "sqlite3.h"
#include "sha256.h"

static int callback(void *NotUsed, int argc, char **argv, char **azColName){
	int i;
	if (NotUsed) {
		fprintf(stderr, "sqlite3 callback unused = %p\n", NotUsed);
	}
	for(i=0; i<argc; i++){
		printf("%s = %s\n", azColName[i], argv[i] ? argv[i] : "NULL");
	}
	printf("\n");
	return 0;
}

struct dbh {
	sqlite3 *db;
	char *errmsg;
	int rc;
};

#define SQLERROR(x) fprintf(stderr, "%s %d: %s\n", __func__, __LINE__, (x))
int begin(sqlite3 *db) {
	int rc;
	char *err;

	rc = sqlite3_exec(db, "begin;", callback, 0, &err);
	if (rc != SQLITE_OK) {
		SQLERROR(err);
		sqlite3_free(err);
	}
	return rc;
}

int commit(sqlite3 *db) {
	int rc;
	char *err;

	rc = sqlite3_exec(db, "commit;", callback, 0, &err);
	if (rc != SQLITE_OK) {
		SQLERROR(err);
		sqlite3_free(err);
	}
	return rc;
}

int rollback(sqlite3 *db) {
	int rc;
	char *err;

	rc = sqlite3_exec(db, "rollback;", callback, 0, &err);
	if (rc != SQLITE_OK) {
		SQLERROR(err);
		sqlite3_free(err);
	}
	return rc;
}
